Type: Package
Package: mlr3mbo
Title: Flexible Bayesian Optimization
Version: 1.2.0
Authors@R: c(
    person("Marc", "Becker", , "marcbecker@posteo.de", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-8115-0400")),
    person("Lennart", "Schneider", , "lennart.sch@web.de", role = "aut",
           comment = c(ORCID = "0000-0003-4152-5308")),
    person("Jakob", "Richter", , "jakob1richter@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-4481-5554")),
    person("Michel", "Lang", , "michellang@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-9754-0393")),
    person("Bernd", "Bischl", , "bernd_bischl@gmx.net", role = "aut",
           comment = c(ORCID = "0000-0001-6002-6980")),
    person("Florian", "Pfisterer", , "pfistererf@googlemail.com", role = "aut",
           comment = c(ORCID = "0000-0001-8867-762X")),
    person("Martin", "Binder", , "mlr.developer@mb706.com", role = "aut"),
    person("Sebastian", "Fischer", , "sebf.fischer@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-9609-3197")),
    person("Michael H.", "Buselli", role = "cph"),
    person("Wessel", "Dankers", role = "cph"),
    person("Carlos", "Fonseca", role = "cph"),
    person("Manuel", "Lopez-Ibanez", role = "cph"),
    person("Luis", "Paquete", role = "cph"))
Description: A modern and flexible approach to Bayesian Optimization / Model
    Based Optimization building on the 'bbotk' package. 'mlr3mbo' is a toolbox
    providing both ready-to-use optimization algorithms as well as their fundamental
    building blocks allowing for straightforward implementation of custom
    algorithms. Single- and multi-objective optimization is supported as well as
    mixed continuous, categorical and conditional search spaces. Moreover, using
    'mlr3mbo' for hyperparameter optimization of machine learning models within the
    'mlr3' ecosystem is straightforward via 'mlr3tuning'. Examples of ready-to-use
    optimization algorithms include Efficient Global Optimization by Jones et al.
    (1998) <doi:10.1023/A:1008306431147>, ParEGO by Knowles (2006)
    <doi:10.1109/TEVC.2005.851274> and SMS-EGO by Ponweiser et al. (2008)
    <doi:10.1007/978-3-540-87700-4_78>.
License: LGPL-3
URL: https://mlr3mbo.mlr-org.com, https://github.com/mlr-org/mlr3mbo
BugReports: https://github.com/mlr-org/mlr3mbo/issues
Depends: mlr3 (>= 1.2.0), mlr3tuning (>= 1.4.0), R (>= 3.1.0)
Imports: bbotk (>= 1.8.0), checkmate (>= 2.0.0), data.table, lgr (>=
        0.3.4), mlr3misc (>= 0.21.0), paradox (>= 1.0.1), spacefillr,
        R6 (>= 2.4.1)
Suggests: emoa, DiceKriging, fastGHQuad, lhs, mlr3learners (>= 0.12.0),
        mirai, mlr3pipelines (>= 0.5.2), moocore, nloptr, processx,
        ranger, rgenoud, rpart, redux, rush (>= 1.0.0), stringi,
        testthat (>= 3.0.0)
ByteCompile: no
Encoding: UTF-8
Config/testthat/edition: 3
Config/testthat/parallel: false
NeedsCompilation: yes
Collate: 'mlr_acqfunctions.R' 'AcqFunction.R' 'AcqFunctionAEI.R'
        'AcqFunctionCB.R' 'AcqFunctionEHVI.R' 'AcqFunctionEHVIGH.R'
        'AcqFunctionEI.R' 'AcqFunctionEILog.R' 'AcqFunctionEIPS.R'
        'AcqFunctionMean.R' 'AcqFunctionMulti.R' 'AcqFunctionPI.R'
        'AcqFunctionSD.R' 'AcqFunctionSmsEgo.R'
        'AcqFunctionStochasticCB.R' 'AcqFunctionStochasticEI.R'
        'AcqOptimizer.R' 'mlr_acqoptimizers.R' 'AcqOptimizerDirect.R'
        'AcqOptimizerLbfgsb.R' 'AcqOptimizerLocalSearch.R'
        'AcqOptimizerRandomSearch.R' 'mlr_input_trafos.R'
        'InputTrafo.R' 'InputTrafoUnitcube.R' 'aaa.R' 'OptimizerADBO.R'
        'OptimizerMbo.R' 'OptimizerAsyncMbo.R' 'mlr_output_trafos.R'
        'OutputTrafo.R' 'OutputTrafoLog.R' 'OutputTrafoStandardize.R'
        'mlr_result_assigners.R' 'ResultAssigner.R'
        'ResultAssignerArchive.R' 'ResultAssignerSurrogate.R'
        'Surrogate.R' 'SurrogateLearner.R'
        'SurrogateLearnerCollection.R' 'TunerADBO.R' 'TunerAsyncMbo.R'
        'TunerMbo.R' 'mlr_loop_functions.R' 'bayesopt_ego.R'
        'bayesopt_emo.R' 'bayesopt_mpcl.R' 'bayesopt_parego.R'
        'bayesopt_smsego.R' 'bibentries.R' 'conditions.R' 'helper.R'
        'loop_function.R' 'mbo_defaults.R' 'sugar.R' 'zzz.R'
Config/roxygen2/version: 8.0.0.9000
Packaged: 2026-07-21 09:38:49 UTC; marc
Author: Marc Becker [cre, aut] (ORCID: <https://orcid.org/0000-0002-8115-0400>),
  Lennart Schneider [aut] (ORCID:
    <https://orcid.org/0000-0003-4152-5308>),
  Jakob Richter [aut] (ORCID: <https://orcid.org/0000-0003-4481-5554>),
  Michel Lang [aut] (ORCID: <https://orcid.org/0000-0001-9754-0393>),
  Bernd Bischl [aut] (ORCID: <https://orcid.org/0000-0001-6002-6980>),
  Florian Pfisterer [aut] (ORCID:
    <https://orcid.org/0000-0001-8867-762X>),
  Martin Binder [aut],
  Sebastian Fischer [aut] (ORCID:
    <https://orcid.org/0000-0002-9609-3197>),
  Michael H. Buselli [cph],
  Wessel Dankers [cph],
  Carlos Fonseca [cph],
  Manuel Lopez-Ibanez [cph],
  Luis Paquete [cph]
Maintainer: Marc Becker <marcbecker@posteo.de>
Repository: CRAN
Date/Publication: 2026-07-21 10:30:20 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-21 12:06:16 UTC; unix
Archs: mlr3mbo.so.dSYM
