# Generated by roxygen2: do not edit by hand

S3method(as.data.table,DictionaryAcqFunction)
S3method(as.data.table,DictionaryAcqOptimizer)
S3method(as.data.table,DictionaryInputTrafo)
S3method(as.data.table,DictionaryLoopFunction)
S3method(as.data.table,DictionaryOutputTrafo)
S3method(as.data.table,DictionaryResultAssigner)
S3method(print,loop_function)
export(AcqFunction)
export(AcqFunctionAEI)
export(AcqFunctionCB)
export(AcqFunctionEHVI)
export(AcqFunctionEHVIGH)
export(AcqFunctionEI)
export(AcqFunctionEILog)
export(AcqFunctionEIPS)
export(AcqFunctionMean)
export(AcqFunctionMulti)
export(AcqFunctionPI)
export(AcqFunctionSD)
export(AcqFunctionSmsEgo)
export(AcqFunctionStochasticCB)
export(AcqFunctionStochasticEI)
export(AcqOptimizer)
export(AcqOptimizerDirect)
export(AcqOptimizerLbfgsb)
export(AcqOptimizerLocalSearch)
export(AcqOptimizerRandomSearch)
export(InputTrafo)
export(InputTrafoUnitcube)
export(OptimizerADBO)
export(OptimizerAsyncMbo)
export(OptimizerMbo)
export(OutputTrafo)
export(OutputTrafoLog)
export(OutputTrafoStandardize)
export(ResultAssigner)
export(ResultAssignerArchive)
export(ResultAssignerSurrogate)
export(Surrogate)
export(SurrogateLearner)
export(SurrogateLearnerCollection)
export(TunerADBO)
export(TunerAsyncMbo)
export(TunerMbo)
export(acqf)
export(acqfs)
export(acqo)
export(bayesopt_ego)
export(bayesopt_emo)
export(bayesopt_mpcl)
export(bayesopt_parego)
export(bayesopt_smsego)
export(default_acqfunction)
export(default_acqoptimizer)
export(default_gp)
export(default_loop_function)
export(default_result_assigner)
export(default_rf)
export(default_surrogate)
export(error_acq_optimizer)
export(error_random_interleave)
export(error_surrogate_update)
export(it)
export(mlr_acqfunctions)
export(mlr_acqoptimizers)
export(mlr_input_trafos)
export(mlr_loop_functions)
export(mlr_output_trafos)
export(mlr_result_assigners)
export(ot)
export(ras)
export(redis_available)
export(srlrn)
import(bbotk)
import(checkmate)
import(data.table)
import(lgr)
import(mlr3)
import(mlr3misc)
import(mlr3tuning)
import(paradox)
import(spacefillr)
importFrom(R6,R6Class)
importFrom(stats,dnorm)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(utils,bibentry)
useDynLib(mlr3mbo,c_eps_indicator)
useDynLib(mlr3mbo,c_sms_indicator)
