# conditions work

    Code
      bayesopt_ego(instance, surrogate = surrogate, acq_function = acq_function,
        acq_optimizer = acq_optimizer)
    Output
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        Optimizer Error.
      

---

    Code
      bayesopt_ego(instance, surrogate = surrogate, acq_function = acq_function,
        acq_optimizer = acq_optimizer)
    Output
      WARN  [bbotk] Caught the following error: 
      x Surrogate update failed.
      > Class: Mlr3ErrorMboSurrogateUpdate
      Caused by:
        x Surrogate Train Error.
        > Class: Mlr3ErrorLearnerTrain
      
      WARN  [bbotk] Caught the following error: 
      x Surrogate update failed.
      > Class: Mlr3ErrorMboSurrogateUpdate
      Caused by:
        x Surrogate Train Error.
        > Class: Mlr3ErrorLearnerTrain
      
      WARN  [bbotk] Caught the following error: 
      x Surrogate update failed.
      > Class: Mlr3ErrorMboSurrogateUpdate
      Caused by:
        x Surrogate Train Error.
        > Class: Mlr3ErrorLearnerTrain
      
      WARN  [bbotk] Caught the following error: 
      x Surrogate update failed.
      > Class: Mlr3ErrorMboSurrogateUpdate
      Caused by:
        x Surrogate Train Error.
        > Class: Mlr3ErrorLearnerTrain
      
      WARN  [bbotk] Caught the following error: 
      x Surrogate update failed.
      > Class: Mlr3ErrorMboSurrogateUpdate
      Caused by:
        x Surrogate Train Error.
        > Class: Mlr3ErrorLearnerTrain
      
      WARN  [bbotk] Caught the following error: 
      x Surrogate update failed.
      > Class: Mlr3ErrorMboSurrogateUpdate
      Caused by:
        x Surrogate Train Error.
        > Class: Mlr3ErrorLearnerTrain
      

---

    Code
      bayesopt_ego(instance, surrogate = surrogate, acq_function = acq_function,
        acq_optimizer = acq_optimizer)
    Output
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        x Surrogate Predict Error.
        > Class: Mlr3ErrorLearnerPredict
      
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        x Surrogate Predict Error.
        > Class: Mlr3ErrorLearnerPredict
      
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        x Surrogate Predict Error.
        > Class: Mlr3ErrorLearnerPredict
      
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        x Surrogate Predict Error.
        > Class: Mlr3ErrorLearnerPredict
      
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        x Surrogate Predict Error.
        > Class: Mlr3ErrorLearnerPredict
      
      WARN  [bbotk] Caught the following error: 
      x Acquisition function optimization failed.
      > Class: Mlr3ErrorMboAcqOptimizer
      Caused by:
        x Surrogate Predict Error.
        > Class: Mlr3ErrorLearnerPredict
      

