% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mbo_defaults.R
\name{default_acqfunction}
\alias{default_acqfunction}
\title{Default Acquisition Function}
\usage{
default_acqfunction(instance)
}
\arguments{
\item{instance}{(\link[bbotk:OptimInstance]{bbotk::OptimInstance}).
An object that inherits from \link[bbotk:OptimInstance]{bbotk::OptimInstance}.}
}
\value{
\link{AcqFunction}
}
\description{
Chooses a default acquisition function, i.e. the criterion used to propose future points.
For synchronous single-objective optimization and a purely numeric parameter space,
defaults to \link{mlr_acqfunctions_cb} with \code{lambda = 3}.
For synchronous single-objective optimization and a mixed numeric-categorical parameter space,
defaults to \link{mlr_acqfunctions_cb} with \code{lambda = 1}.
For synchronous multi-objective optimization, defaults to \link{mlr_acqfunctions_smsego}.
For asynchronous single-objective optimization, defaults to \link{mlr_acqfunctions_stochastic_cb}.
}
\seealso{
Other mbo_defaults: 
\code{\link{default_acqoptimizer}()},
\code{\link{default_gp}()},
\code{\link{default_loop_function}()},
\code{\link{default_result_assigner}()},
\code{\link{default_rf}()},
\code{\link{default_surrogate}()},
\code{\link{mbo_defaults}}
}
\concept{mbo_defaults}
